@@ -3,6 +3,14 @@
 setAs("dpoMatrix", "dppMatrix",
       function(from) as(as(from, "dspMatrix"), "dppMatrix"))
 
+setAs("dpoMatrix", "correlation",
+      function(from) {
+          sd <- sqrt(diag(from))
+          if (!is.null(nms <- from@Dimnames[[1]])) names(sd) <- nms
+          ans <- new("correlation", as(t(from/sd)/sd, "dpoMatrix"), sd = sd)
+          ans@Dimnames <- from@Dimnames
+          ans
+      })
 
 setMethod("chol", signature(x = "dpoMatrix"),
           function(x, pivot, LINPACK)
@@ -38,3 +46,4 @@ setMethod("solve", signature(a = "dpoMatrix", b = "matrix"),
 ##          as.numeric(.Call("dpoMatrix_matrix_solve",
 ##                           a, as.matrix(b), PACKAGE = "Matrix")),
 ##          valueClass = "numeric")
+
