@@ -44,32 +44,19 @@ SEXP dpoMatrix_chol(SEXP x)
     return set_factors(x, val, "Cholesky");
 }
 
-static
-double set_rcond(SEXP obj, char *typstr)
-{
-    char typnm[] = {'O', '\0'};	/* always use the one norm */
-    SEXP rcv = GET_SLOT(obj, Matrix_rcondSym);
-    double rcond = get_double_by_name(rcv, typnm);
-
-    if (R_IsNA(rcond)) {
-        SEXP Chol = dpoMatrix_chol(obj);
-	int *dims = INTEGER(GET_SLOT(Chol, Matrix_DimSym)), info;
-	double anorm = get_norm_sy(obj, typnm);
-
-	F77_CALL(dpocon)(uplo_P(Chol),
-			 dims, REAL(GET_SLOT(Chol, Matrix_xSym)),
-			 dims, &anorm, &rcond,
-			 (double *) R_alloc(3*dims[0], sizeof(double)),
-			 (int *) R_alloc(dims[0], sizeof(int)), &info);
-	SET_SLOT(obj, Matrix_rcondSym,
-		 set_double_by_name(rcv, rcond, typnm));
-    }
-    return rcond;
-}
-
 SEXP dpoMatrix_rcond(SEXP obj, SEXP type)
 {
-    return ScalarReal(set_rcond(obj, CHAR(asChar(type))));
+    SEXP Chol = dpoMatrix_chol(obj);
+    char typnm[] = {'O', '\0'};	/* always use the one norm */
+    int *dims = INTEGER(GET_SLOT(Chol, Matrix_DimSym)), info;
+    double anorm = get_norm_sy(obj, typnm), rcond;
+
+    F77_CALL(dpocon)(uplo_P(Chol),
+		     dims, REAL(GET_SLOT(Chol, Matrix_xSym)),
+		     dims, &anorm, &rcond,
+		     (double *) R_alloc(3*dims[0], sizeof(double)),
+		     (int *) R_alloc(dims[0], sizeof(int)), &info);
+    return ScalarReal(rcond);
 }
 
 SEXP dpoMatrix_solve(SEXP x)
@@ -82,9 +69,10 @@ SEXP dpoMatrix_solve(SEXP x)
     SET_SLOT(val, Matrix_uploSym, duplicate(GET_SLOT(Chol, Matrix_uploSym)));
     SET_SLOT(val, Matrix_xSym, duplicate(GET_SLOT(Chol, Matrix_xSym)));
     SET_SLOT(val, Matrix_DimSym, duplicate(GET_SLOT(Chol, Matrix_DimSym)));
+    SET_SLOT(val, Matrix_DimNamesSym,
+	     duplicate(GET_SLOT(x, Matrix_DimNamesSym)));
     F77_CALL(dpotri)(uplo_P(val), dims,
 		     REAL(GET_SLOT(val, Matrix_xSym)), dims, &info);
-    SET_SLOT(val, Matrix_rcondSym, duplicate(GET_SLOT(x, Matrix_rcondSym)));
     UNPROTECT(1);
     return val;
 }
@@ -99,7 +87,6 @@ SEXP dpoMatrix_dgeMatrix_solve(SEXP a, SEXP b)
 
     if (*adims != *bdims || bdims[1] < 1 || *adims < 1)
 	error(_("Dimensions of system to be solved are inconsistent"));
-    SET_SLOT(val, Matrix_rcondSym, allocVector(REALSXP, 0));
     SET_SLOT(val, Matrix_factorSym, allocVector(VECSXP, 0));
     SET_SLOT(val, Matrix_DimSym, duplicate(GET_SLOT(b, Matrix_DimSym)));
     SET_SLOT(val, Matrix_xSym, duplicate(GET_SLOT(b, Matrix_xSym)));
